# bscount.cl -- estimate Bahcall & Soneira star counts for given (l,b)
#
# Taken from Appendix B of Bahcall & Soneira 1980, ApJS 44, 73. (B&S80)
#
# 950807: created
# ...might eventually add capability to do multiple fields/mag ranges at once,
#	taking input from a file perhaps.

procedure bscount (long,lat,mag,filter,option,xtinct)

real   long {min=0.,max=360.,prompt="Galactic longitude (l) of field"}
real   lat {min=-90.,max=90.,prompt="Galactic latitude (b) of field"}
real   mag {prompt="Apparent magnitude at which to calculate counts"}
string filter {enum="B|V",prompt="Filter in which to calculate counts"}
string option {enum="differential|integral",prompt="Differential or integral counts?"}
real   xtinct=0.0 {min=0.,prompt="Extinction (obscuration), in magnitudes"}
bool   verbose {prompt="Verbose output? (mag, counts, l, b, ext., filter, option)"}

begin

	real l, b, m, extinct, mtmp, pi
	real c1, beta, mstar, alpha, mu, gamma, delta
	real c2, eta, mcross, kappa, lambda, sigma
	real T1a, T1b, T1, T2, T2a, T2b, T, A
	string ffilter, ooption
	bool verb

pi=3.1415926
#l=long*2*pi/360.
#b=lat*2*pi/360.
l=abs(long*2*pi/360.)
b=abs(lat*2*pi/360.)
mtmp=mag
ffilter=filter
ooption=option
extinct=xtinct
m=mtmp-extinct
verb=verbose

#set up magnitude-dependent constants
if(m<=12){
	mu=0.03
	gamma=0.36
	} else {
	if(m>=20){
		mu=0.09
		gamma=0.04
	} else {
		mu=0.0075*(m-12)+0.03
		gamma=0.04*(12-m)+0.36
	}
}

#set up option-dependent constants
#differential V counts, stars mag-1 deg-2
if(ffilter=="V" && ooption=="differential"){
	c1=200
	alpha=-0.2
	beta=0.01
	delta=2
	mstar=15
	c2=400
	kappa=-0.26
	eta=0.065
	lambda=1.5
	mcross=17.5
}

#integral V counts, stars deg-2
if(ffilter=="V" && ooption=="integral"){
	c1=925
	alpha=-0.132
	beta=0.035
	delta=3.0
	mstar=15.75
	c2=1050
	kappa=-0.180
	eta=0.087
	lambda=2.50
	mcross=17.5
}

#differential B counts, stars mag-1 deg-2
if(ffilter=="B" && ooption=="differential"){
	c1=235
	alpha=-0.227
	beta=0.0
	delta=1.50
	mstar=17
	c2=370
	kappa=-0.175
	eta=0.06
	lambda=2.0
	mcross=18
}

#integral B counts, stars deg-2
if(ffilter=="B" && ooption=="integral"){
	c1=950
	alpha=-0.124
	beta=0.027
	delta=3.1
	mstar=16.60
	c2=910
	kappa=-0.167
	eta=0.083
	lambda=2.50
	mcross=18
}

#do the calculation. T is from eq. B1 of B&S80.
T1a=(c1*10**(beta*(m-mstar)))/((1+10**(alpha*(m-mstar)))**delta)

#T1b=((sin(b)*(1-mu*cos(l)/abs(tan(b))))**(3-5*gamma))**(-1.0)
T1b=((sin(b)*(1-mu*cos(l)/(tan(b))))**(3-5*gamma))**(-1.0)

T1=T1a*T1b

T2a=(c2*10**(eta*(m-mcross)))/((1+10**(kappa*(m-mcross)))**lambda)

#T2b=((1-abs(cos(b))*cos(l))**(1.45-0.20*abs(cos(b))*cos(l)))**(-1.0)
T2b=((1-(cos(b))*cos(l))**(1.45-0.20*(cos(b))*cos(l)))**(-1.0)

T2=T2a*T2b

T=T1+T2

if(verb){
		print(m//" "//T//" "//long//" "//lat//" "//extinct//" "//ffilter//" "//ooption)
	} else {
		print(T)
	}

end

