procedure btc_chipscales(filename,scalename)

# btc_chipscales -- find and/or apply BTC chip-to-chip photometric scalings 

# This program will fail if the ITERMEAN keyword is not in all image headers!

# 981113: created from btc_getchipscales and btc_dochipscales

string  filename	{prompt="File of root names of BTC image sets"}
string  scalename	{prompt="Root name for output scaling files"}
string  extension="fits" {prompt="Image extension"}
bool   	getscales      	{yes,prompt="Get individual image set scalings?"}
bool   	avgscales      	{yes,prompt="Average lots of scalings together?"}
bool   	savescales     	{yes,prompt="Accept and save average scaling?",mode="q"}
bool   	applyscales    	{yes,prompt="Apply scalings to the images?"}
bool    doiterstat      {no,prompt="Rerun iterstat on the final scaled images?"}
bool   	echog        	{yes,prompt="Beep when task is done?"}
struct  *name

begin

string	root, image, scales, extn, scale_name, base, scale_file
real 	avgmean, chipscale, dvone, dvtwo, dvthree, dvfour, divisor
bool	get_scales, avg_scales, save_scales, apply_scales, redoiterstat

name = filename
scale_name = scalename
extn = extension
get_scales = getscales
avg_scales = avgscales
apply_scales = applyscales
redoiterstat = doiterstat

# -- cycle through images
if (get_scales) {
    print ("Output to "//scale_name//"_chipscales:")
    while (fscan (name, root) != EOF) {

# -- find average itermean
	hsel (root//"_?."//extn, "itermean", yes) | average | scan (avgmean)

# -- get itermean for all 4 images in the set and divide by average itermean
	scales=""
	for (i=1; i<=4; i+=1) {
		image = root//"_"//i//"."//extn
		hsel (image, "itermean", yes) | scan (chipscale)
		chipscale=chipscale/avgmean
		scales = (scales//" "//chipscale)
	}

# -- output
	print (root//scales)
	print (root//scales, >> scale_name//"_chipscales")
    }
}

# -- average lots of image set scalings together chip-by-chip
while (avg_scales) {
 print ("Average scalings, RMS uncertainty and # of images used for chips 1-4:")
 fields (scale_name//"_chipscales",2) | iteravg >> scale_name//"_chipscalesavg")
 fields (scale_name//"_chipscales",3) | iteravg >> scale_name//"_chipscalesavg")
 fields (scale_name//"_chipscales",4) | iteravg >> scale_name//"_chipscalesavg")
 fields (scale_name//"_chipscales",5) | iteravg >> scale_name//"_chipscalesavg")
 type (scale_name//"_chipscalesavg")
 print (" ")
 print ("If the above is unacceptable, answer `n' and the "//scale_name//"_chipscales")
 print ("file will be displayed for editing and the iterative average re-run.")
 save_scales=savescales
 if(save_scales){
 	print ("Saving to file "//scale_name//"_chipscalesavg.")
   print ("Additions to this file should ONLY be as comments at the end of it!")
   print ("#--> Any additions to this file MUST be placed after this line! <--", >> scale_name//"_chipscalesavg")
	avg_scales=no
 } else {
	avg_scales=yes
 	del (scale_name//"_chipscalesavg", ver-)
	vi (scale_name//"_chipscales")
 }
}

# -- apply chip-to-chip photometric scalings to btc image sets
if (apply_scales) {
    scale_file = scale_name//"_chipscalesavg"
    if(access(scale_file)) {
	dvone=0.; dvtwo=0.; dvthree=0.; dvfour=0.
    	fields (scale_file, "1", lines="1") | scan (dvone)
    	fields (scale_file, "1", lines="2") | scan (dvtwo)
    	fields (scale_file, "1", lines="3") | scan (dvthree)
    	fields (scale_file, "1", lines="4") | scan (dvfour)
	if ((dvone==0.) || (dvtwo==0.) || (dvthree==0.) || (dvfour==0.)) {
	    beep
    	    print ("Problem reading file "//scale_file//" -- no images scaled!")
	}
    } else {
	beep
    	print ("File "//scale_file//" not found -- no images scaled!")
	apply_scales=no
    }
}

if (apply_scales) {
# -- cycle through images
  name = filename
  while (fscan (name, base) != EOF) {

# -- do the scaling
    for (i=1; i<=4; i+=1) {
	image = base//"_"//i
	if(i==1){
		imgets (image, "chipscal", >& "dev$null")
                if ((imgets.value)=="0") {
			print ("Dividing "//image//" by "//dvone)
			imar (image, "/", dvone, image, ver-, noact-)
			hedit(image,"chipscal",dvone,add+,ver-,show+,update+)
# -- update the iterstat parameters in the image headers
	if (redoiterstat) {
	  print ("Rerunning iterstat on "//image)
	  iterstat (image, print+, ver-, addh+)
	} else {
	  hedit(image,"itermean","(itermean/"//dvone//")",add+,ve-,sho-,updat+)
	  hedit(image,"itermode","(itermode/"//dvone//")",add+,ve-,sho-,updat+)
	  hedit(image,"itersig","(itersig/"//dvone//")",add+,ve-,sho-,updat+)
	  hedit(image,"itermed","(itermed/"//dvone//")",add+,ve-,sho-,updat+)
	  hedit(image,"variance","(variance/("//dvone//"*"//dvone//"))",add+,ve-,sho-,updat+)
	}
		} else {
			print (image//" already divided by "//imgets.value)
		}
	}
	if(i==2){
		imgets (image, "chipscal", >& "dev$null")
                if ((imgets.value)=="0") {
			print ("Dividing "//image//" by "//dvtwo)
			imar (image, "/", dvtwo, image, ver-, noact-)
			hedit(image,"chipscal",dvtwo,add+,ver-,show+,update+)
# -- update the iterstat parameters in the image headers
	if (redoiterstat) {
	  print ("Rerunning iterstat on "//image)
	  iterstat (image, print+, ver-, addh+)
	} else {
	  hedit(image,"itermean","(itermean/"//dvtwo//")",add+,ve-,sho-,updat+)
	  hedit(image,"itermode","(itermode/"//dvtwo//")",add+,ve-,sho-,updat+)
	  hedit(image,"itersig","(itersig/"//dvtwo//")",add+,ve-,sho-,updat+)
	  hedit(image,"itermed","(itermed/"//dvtwo//")",add+,ve-,sho-,updat+)
	  hedit(image,"variance","(variance/("//dvtwo//"*"//dvtwo//"))",add+,ve-,sho-,updat+)
	}
		} else {
			print (image//" already divided by "//imgets.value)
		}
	}
	if(i==3){
		imgets (image, "chipscal", >& "dev$null")
                if ((imgets.value)=="0") {
			print ("Dividing "//image//" by "//dvthree)
			imar (image, "/", dvthree, image, ver-, noact-)
			hedit(image,"chipscal",dvthree,add+,ver-,show+,update+)
# -- update the iterstat parameters in the image headers
	if (redoiterstat) {
	  print ("Rerunning iterstat on "//image)
	  iterstat (image, print+, ver-, addh+)
	} else {
	  hedit(image,"itermean","(itermean/"//dvthree//")",ad+,ve-,sho-,updat+)
	  hedit(image,"itermode","(itermode/"//dvthree//")",ad+,ve-,sho-,updat+)
	  hedit(image,"itersig","(itersig/"//dvthree//")",add+,ve-,sho-,updat+)
	  hedit(image,"itermed","(itermed/"//dvthree//")",add+,ve-,sho-,updat+)
	  hedit(image,"variance","(variance/("//dvthree//"*"//dvthree//"))",add+,ve-,sho-,updat+)
	}
		} else {
			print (image//" already divided by "//imgets.value)
		}
	}
	if(i==4){
		imgets (image, "chipscal", >& "dev$null")
                if ((imgets.value)=="0") {
			print ("Dividing "//image//" by "//dvfour)
			imar (image, "/", dvfour, image, ver-, noact-)
			hedit(image,"chipscal",dvfour,add+,ver-,show+,update+)
# -- update the iterstat parameters in the image headers
	if (redoiterstat) {
	  print ("Rerunning iterstat on "//image)
	  iterstat (image, print+, ver-, addh+)
	} else {
	  hedit(image,"itermean","(itermean/"//dvfour//")",add+,ve-,sho-,updat+)
	  hedit(image,"itermode","(itermode/"//dvfour//")",add+,ve-,sho-,updat+)
	  hedit(image,"itersig","(itersig/"//dvfour//")",add+,ve-,sho-,updat+)
	  hedit(image,"itermed","(itermed/"//dvfour//")",add+,ve-,sho-,updat+)
	  hedit(image,"variance","(variance/("//dvfour//"*"//dvfour//"))",add+,ve-,sho-,updat+)
	}
		} else {
			print (image//" already divided by "//imgets.value)
		}
	}
    }
  }
}

if(echog)beep

end


