# btc_getshifts.cl -- auto-detect stars and find offsets for BTC images.
#
# Task calls following other tasks: fileroot, filecalc.
#
# Output:
# prefix.refcoords.suffix	registration objects' reference image coords
# prefix.allcoords.suffix	registration objects' coordinates in all images
# prefix.offsets.suffix		final shifts logfile, + coarse and final shifts
# prefix.imshifts.suffix	image names and rounded int. shifts on same line
# pprefix.shiftexact.ssuffix	exact final shifts
# prefix.shiftfrac.suffix	fractional part of final shifts
# prefix.shiftint.suffix	integer part of final shifts
# prefix.starfind.suffix	output of starfind on reference image
# prefix.roundoff.suffix	final shifts rounded off to integer shifts
# (optional):
# @flatlist//suffix.imh		fractionally-shifted images
# flatlist//suffix		list of fractionally-shifted images
# 
# CHANGES:
# 990530: re-created from phiirs$irshift.cl, adding auto-detection of stars

procedure btc_getshifts(flatlist,refimage,prefix,suffix)

string flatlist	      {prompt="List of images for which to find offsets (no @)"}
string refimage	      {prompt="Name of (flattened) reference image"}
string prefix	      {prompt="Prefix string for output files"}
string suffix	      {prompt="Suffix string for output files"}

real seeing	      {prompt="Seeing FWHM (pixels) for reference image"}
bool fracshift=no     {prompt="Create fractional-pixel-offset images?"}
string interp         {"poly5",enum="nearest|linear|poly3|poly5|spline3",prompt="Interpolation type"}
string boundary       {"nearest",enum="constant|nearest|reflect|wrap",prompt="Boundary type"}
string badmask=""     {prompt="Bad pixel mask for shifted images (blank=old mask)",mode="q"}
bool useimages=yes    {prompt="Measure shifts from images instead of a file?"}
bool displayim=yes    {prompt="Display images for imexam?"}
bool ptv=yes          {prompt="Use phat.ptv instead of display in imexam?"}
string imsection=""   {prompt="Image section to display in imexam?"}
bool editfiles=yes    {prompt="Display coords files for editing?"}
string inputlist      {prompt="File with coordinate info", mode="q"}
real xshiftold	      {prompt="Reference image x shift in earlier dataset"}
real yshiftold	      {prompt="Reference image y shift in earlier dataset"}
real radius=5.	      {prompt="Imexamine aperture photometry radius"}
real buffer=10.	      {prompt="Imexamine sky buffer"}
real width=10.	      {prompt="Imexamine sky annulus width"}
real bs=9.	      {prompt="Centering box size for final shift calculation"}
bool   echog        {yes,prompt="Beep when task is done?"}
struct *iinputlist
struct *list

begin

string rrefimage, fflatlist, rrefcoords, aallcoords, bbadmask, rroundname, ext
string sshiftlist, coarsefile, ooffname, ssuffix, iintname, ffracname, ooptname
string pprefix, logoffname, sl, sx, img, iinterp, bboundary, im_section
string sstarfind
real p, pixscale, stcoordx, stcoordy, nrows, ncols, delx, dely, adelx, adely
real xold, yold, bbs, xxshiftold, yyshiftold, xshiftnew, yshiftnew, astcoordx
real astcoordy, nl, nx, x, y, xoff, yoff, hwhm, threshold
real new_x, new_y, old_x, old_y, exact_x, exact_y, step, step_x, step_y, see_ing
int xref, yref, int_x, int_y
bool uuseimages, ffracshift, go, ddisplayim, edit_files

# The "go" parameter won't let the task run if you haven't loaded the ctio pkg.
go=yes
if(!deftask("filecalc")) {go=no}
if(!deftask("fileroot")) {go=no}
if(ptv && !deftask("ptv")) {go=no}
if(go) {

fflatlist = flatlist
rrefimage = refimage
pprefix = prefix
ssuffix = suffix

ffracshift = fracshift
	if(ffracshift)bbadmask=badmask
uuseimages = useimages
ddisplayim = displayim
im_section = imsection
edit_files = editfiles
bbs = bs

ooffname = pprefix//".shiftexact."//ssuffix
logoffname = pprefix//".offsets."//ssuffix
rrefcoords = pprefix//".refcoords."//ssuffix
aallcoords = pprefix//".allcoords."//ssuffix
coarsefile = pprefix//".coarse."//ssuffix
iintname = pprefix//".shiftint."//ssuffix
ffracname = pprefix//".shiftfrac."//ssuffix
rroundname = pprefix//".roundoff."//ssuffix
sstarfind = pprefix//".starfind."//ssuffix

# Estimate seeing if needed
if(ddisplayim){
	print("\n===> Find the seeing (type 'a') in the reference image using")
	print("===> the best registration object (visible in all images).\n")
	rimexam.radius=radius
	rimexam.buffer=buffer
	rimexam.width=width
	if (ptv) {
		imexam(rrefimage, frame=1, logfile=rrefcoords, keep+, display="ptv(image='$1',frame=$2)")
	} else {
		imexam(rrefimage, frame=1, logfile=rrefcoords, keep+)
	}
}

# Auto-detect stars
if (!access(sstarfind)) {
	see_ing=seeing
	hwhm=see_ing/2.
	imgets (rrefimage, "ITERSIG", >& "dev$null")
	if (imgets.value=="0") {
		iterstat (rrefimage, addh+, print+, ver-)
		imgets (rrefimage, "ITERSIG", >& "dev$null")
	}
	threshold=5*(real(imgets.value))
       print("STARFINDing stars in "//rrefimage//" w/threshold "//threshold)
	starfind (rrefimage, sstarfind, hwhm, threshold)
}
#fields (sstarfind, "1-2", >> rrefcoords)
fields (sstarfind, "1-2") | grep ("-v", "'^$'", >> rrefcoords)

# Add saturated or otherwise missing stars
if(ddisplayim){
	tvmark ("1", rrefcoords, col=204, txsize=1)
	print("\n===> Mark saturated or other stars missed by STARFIND...\n")
	rimexam.radius=radius
	rimexam.buffer=buffer
	rimexam.width=width
	imexam(rrefimage, logfile=rrefcoords, keep+)
}

# First method: ID objects from images
if(uuseimages){
# ... interactively ...
 if(ddisplayim){
  print("\n===> Find the best registration object marked earlier and hit 'a' to")
  print("===> get its position in each frame.  Hit 'n' to go to the next frame")
  print("===> and 'q' when done.  The file containing these positions")
  print("===> will be displayed for editing when done, in case of mistakes.\n")
  print("===> It is useful to note images with bad seeing, tracking, etc.\n")
  if (ptv) {
	imexam("@"//fflatlist, frame=1, logfile=aallcoords, keep+, display="ptv(image='$1"//im_section//"',frame=$2)")
  } else {
	imexam("@"//fflatlist, frame=1, logfile=aallcoords, keep+, display="display(image='$1"//im_section//"',frame=$2)")
  }
 } else {
# ... or non-interactively ...
  print("===> Images not displayed.")
 }

 if(edit_files){
  print(" ")
  print("===> In case of mistakes, both coordinate files can now be edited.")
  vi(rrefcoords)
  vi(aallcoords)
 }

 print(" ")
 print("===> Finding the coarse offsets...")
 fields (rrefcoords, "1-2") | scan (xref, yref)
 lintran(aallcoords,x1=0.001,y1=0.001,x2=xref,y2=yref,angle=180, >coarsefile)

}

# Second method: estimate shifts from telescope offsets
if(!uuseimages){
iinputlist=inputlist
print("Generating initial shift file from file "//inputlist//".")
while (fscan (iinputlist, pixscale, astcoordx, astcoordy, nrows, ncols, adelx, adely) !=EOF) {
	stcoordx=astcoordx/pixscale
	stcoordy=astcoordy/pixscale
	delx=adelx/pixscale
	dely=adely/pixscale
	x=stcoordx
	xold=stcoordx
	y=stcoordy
	yold=stcoordy

#procedure:i=1 (0,-dely); i=2 (-delx,+dely); i=3 (-delx,-dely); i=4(-delx,+dely)
	for (i=1; i<=nrows; i+=1) {
		for (j=1; j<=ncols; j+=1) {
			y=yold+dely*(-1)**(i+1)
			print(x, y, >> aallcoords)
			yold=y
			} 
		x=xold+delx
		xold=x
		y=yold+dely*(-1)**(i+1)
		yold=y
		}
	}

print("===> Edit the file with :wq twice.")
	
vi("-c ':g/0\.\ /s//0.00 /g'", aallcoords)
vi("-c ':g/0\.$/s//0.00/g'", aallcoords)

print(" ")
print("===> Type in the reference image offsets (printed 2 lines below) when")
print("===> requested, to zero the coordinate system on the reference image.")

del("tempsortlist", ver-, >& "dev$null")
join(fflatlist, aallcoords, out="tempsortlist")
grep(rrefimage,"tempsortlist")
del("tempsortlist",ver-)

print("===> Finding the coarse offsets...")
lintran(aallcoords, x2=xref, y2=yref, angle=180, > coarsefile)
}

# Common ending for both approaches.
print(" ")
print("===> Finding the final offsets...")
	del(logoffname, ver-, >& "dev$null")
	imcentroid("@"//fflatlist, rrefimage, rrefcoords, shift=coarsefile, boxsize=bbs, > logoffname)
	del("temp.joinfile", ver-, >& "dev$null")
	del("temp.joinfile?", ver-, >& "dev$null")
# read in length of temp.joinfile0 and cut out last two lines
	copy(logoffname,"temp.joinfile0")
	count("temp.joinfile0") | scan (nl)
	nx=nl-2
	head("temp.joinfile0", nlines=nx, >> "temp.joinfile1")
# find line on which the string "X-s" appears and cut out that line & all before
	!grep -n X-s temp.joinfile1 > temp.joinfile2
        type("temp.joinfile2") | scan (sl)
	x=stridx(":",sl)
	sx=substr(sl,1,x)
	x=nx-real(sx)
	tail("temp.joinfile1", nlines=x, >> "temp.joinfile")
# Cut down on number of output files
	print(" ", >> logoffname)
	print("#Coarse offsets:", >> logoffname)
	print(" ", >> logoffname)
	type(coarsefile, >> logoffname)
	del(coarsefile, ver-)

# Create exact shifts list
	fields("temp.joinfile", "2,4", >> ooffname)
	del("temp.joinfile*", ver-)

# Create integer shifts list
	list=ooffname
	while(fscan(list,x,y) !=EOF) {
		print(int(x),int(y), >> iintname)
	}
# Create fractional shifts list
	join(iintname,ooffname,out="temp.joinfile")
        filecalc("temp.joinfile","$3-$1;$4-$2",forma="%6.2f%6.2f", >> ffracname)
	flpr; flpr; flpr

# Create optimum rounded-off integer offsets
print(" ")
print("===> Finding optimum rounded-off integer offsets...")
old_x = 999.; old_y = 999.
# for each 0.01 pixel step...
for (i=-50; i<50; i+=1) {
	step = i / 100.
#	print (i)
# ...recalculate integer offsets...
	list=ooffname
	del (rroundname, ver-, >& "dev$null")
	del ("temp."//rroundname, ver-, >& "dev$null")
        while(fscan(list,xoff,yoff) !=EOF) {
		xoff=xoff+step
		yoff=yoff+step
                if(((abs(xoff))>=0.5) && ((abs(yoff))>=0.5)) {
                        print(xoff,yoff, >> "temp."//rroundname)
                } else {
                        if((abs(xoff))<=0.5) {
                                xoff=0.
                        }
                        if((abs(yoff))<=0.5) {
                                yoff=0.
                        }
                        print(xoff,yoff, >> "temp."//rroundname)
                }
     filecalc("temp."//rroundname, "$1;$2", format="%4.0f %4.0f", >> rroundname)
     		flpr; flpr; flpr
                del ("temp."//rroundname, ver-, >& "dev$null")
        }
        del ("temp."//rroundname, ver-, >& "dev$null")
# calculate sum of squares of residuals of exact offsets - new integer offsets
	filecalc (ooffname//","//rroundname, "$1@1-$1@2+"//step) | average (option = "new_sample", >& "dev$null")
	flpr; flpr; flpr
	new_x=average.sumsqrs
	filecalc (ooffname//","//rroundname, "$2@1-$2@2+"//step) | average (option = "new_sample", >& "dev$null")
	flpr; flpr; flpr
	new_y=average.sumsqrs
	if (new_x < old_x) { 
		old_x = new_x
		step_x = step
	}
	if (new_y < old_y) {
		old_y = new_y
		step_y = step
	}
}
# recalculate final integer offsets
list=ooffname
del(rroundname,ver-)
del ("temp."//rroundname, ver-, >& "dev$null")
while(fscan(list,xoff,yoff) !=EOF) {
	xoff=xoff+step_x
	yoff=yoff+step_y
        if(((abs(xoff))>=0.5) && ((abs(yoff))>=0.5)) {
                print(xoff,yoff, >> "temp."//rroundname)
        } else {
                if((abs(xoff))<=0.5) {
                       xoff=0.
                }
                if((abs(yoff))<=0.5) {
                       yoff=0.
                }
                print(xoff,yoff, >> "temp."//rroundname)
        }
        filecalc("temp."//rroundname,"$1;$2",format="%4.0f %4.0f", >>rroundname)
	flpr; flpr; flpr
        del ("temp."//rroundname, ver-, >& "dev$null")
}
del ("temp."//rroundname, ver-, >& "dev$null")

print(" ", >> logoffname)
print("#Final offsets:", >> logoffname)
print(" ", >> logoffname)
type(ooffname, >> logoffname)
del("temp.joinfile", ver-)
join(fflatlist,rroundname,out=pprefix//".imshifts."//ssuffix)

# Produce photometric time series from *allcoords* file
fields(aallcoords, "7") | graph (title="Flux of centering object as f(image number)", point-)

# Create fractional-pixel-shifted images
if(ffracshift){
	sshiftlist=fflatlist//ssuffix
	print(" ")
	print("===> Creating fractional-pixel-shifted images listed in "//sshiftlist//".")
	list=fflatlist
	while(fscan(list,img) !=EOF) {
		fileroot(img,validim+)
		img=fileroot.root
		ext=fileroot.extension
		if(ext!="")ext="."//ext
		print(img//ssuffix//ext, >> sshiftlist)
	}
	iinterp=interp
	bboundary=boundary
	imshift("@"//fflatlist, "@"//sshiftlist, shifts=ffracname, interp=iinterp, bound=bboundary)
	if(bbadmask!="") {
		hedit("@"//sshiftlist,"BPM",bbadmask,add+,ver-,show-,update+)
		hedit("@"//sshiftlist,"BPMASK",bbadmask,add+,ver-,show-,update+)
	}
}

print("...it is accomplished.  Next you should run `ircoadd' using")
if(ffracshift)print(sshiftlist//" and "//iintname//" or")
print(fflatlist//" and "//rroundname//".")

} else {
   print("WARNING: tasks `filecalc', `fileroot', or package `phat' (if ptv=yes)"
	print("are not loaded!  Please load them and restart this task.")
}

if(echog)beep

end

