procedure complete(image1,image2,image3,untrim1,untrim2,untrim3,mask,sum,color21,color31,mlim1,spacing,nbins,nobjs,nrepeat,pixscale)

# complete.cl -- calculate completeness corrections for mosaiced image
#
# **OUTPUT: comparea_image? and complete_image? where ?=1,2,3 as appropriate
#
# **REQUIRES packages FOCAS, ARTDATA (mkobjects), CTIO (filecalc), 
#   PHIIRS (fileroot), PHAT (imswap),
# **REQUIRES rlambda, ebv, gain, rdnoise, fwhmpix, & zeropt in each image header
# **REQUIRES existing image1.cat, image2.cat, image3.cat, & sum.cat
#
# 970731: working version finally debugged, by Pat Hall.
# 970806: added insertion of xyminmax, and fixed minor related inefficiency bug
#
# **comparea_image? output will be ERRONEOUS unless mag limits input to the
#   task bracket all mlims which exist in the image(s).
# **CANNOT BE RUN IN BATCH MODE due to focas.mfilter/ffilter problem.
#
# --could replace bright magnitude C(m) with (sum(areas,objs<m)/Atot),
#	but this procedure should calculate that a bit better due to splitting
#	of previously-unsplit objs, albeit a bit noiser due to finite # of objs
# --could extend to completeness matrix, not just C(m)
# --could extend to a range of colors, not just an avg. color

string image1 	{prompt="K (1st filter) image & catalog root name"}
string image2 	{prompt="r (2nd filter) image & catalog root name (optional)"}
string image3 	{prompt="Other filter image & catalog root name (optional)"}
string untrim1 	{prompt="K (1st filter) area as f(m_lim) image name"}
string untrim2 	{prompt="r (2nd filter) area as f(m_lim) image name (optional)"}
string untrim3 	{prompt="Other filter area as f(m_lim) image name (optional)"}
string mask 	{prompt="Mask image from which to find x,y limits"}
string sum 	{prompt="Root name of comparison summed image"}
real color21=0 {prompt="Typical r-K (2nd - 1st filter) galaxy color (optional)"}
real color31=0  {prompt="Typical (3rd - 1st filter) galaxy color (optional)"}
real mlim1=0	{prompt="Magnitude of faintest bin in K (1st filter)"}
#real mlim2=0	{prompt="Magnitude of faintest bin in r (2nd filter; optional)"}
#real mlim3=0	{prompt="Magnitude of faintest bin in other filter; optional"}
real spacing  {prompt="Magnitude bin spacing (should evenly divide color21,31)"}
int nbins   	{prompt="Number of magnitude (flux) bins to create"}
int nobjs	{prompt="Number of objects to simultaneously add to images"}
int nrepeat    {prompt="Number of times to add nobjs to image at each flux bin"}
real pixscale   {prompt="Scale of images in arcsec/pixel"}

bool disimages  {prompt="Display temporary cleaned and altered images?"}
string log_file {prompt="(Optional) name of verbose log file"}
int border=5	{min=1,prompt="Size in pixels of FOCAS catalog border region"}
int cleanfactor=1 {min=1,prompt="Area to increase FOCAS isophotes for cleaning"}
real firstseed=1.  {prompt="Starting seed integer for ARTDATA object positions"}
string psf="moffat"	{enum="gaussian|moffat|image|textfile",prompt="Type of PSF for ARTDATA"}
bool   echog        {yes,prompt="Beep when task is done?"}
struct *list

begin

string	img, msk, img1, img2, img3, f, s1, s2, s3, s4, s5, ssum, sstar
string  norm, norm1, norm2, norm3, logname
real	rlambda1, alambda1, rlambda2, alambda2, rlambda3, alambda3, mag1, mag2
real	sspacing, llimbin, f1, f2, f3, xlong, ylong, factor, mag, rn, ffwhmpix
real	ebv, rlambda, alambda, xmin, xmax, ymin, ymax, x, y, z, sseed, zeropt
real	g, skysig, variance, frecover, mreal, mnorm, Ltotal, deltam, mfinal
real	Atot, diff, mtmp, ctmp, mmean, nnpix, ppixscale, area, Atmp, nrecovsum
real	ctrue, llimbin1, llimbin2, llimbin3, color_21, color_31, mag3, a, b
real	ndenomsum, ndenom, Asum
real    farray[82,4]=82(0),4(0)
real    aarray[82,4]=82(0),4(0)
real    barray[82,4]=82(0),4(0)
int    	nnobjs, nnbins, nnrepeat, i, j, k, l, m, nrecov, deltamax, cf, running
int     index, binmax, bborder, kmax, i1, i2, total, aindex
bool	ddis, go

# Certain tasks/packages must be loaded for task to run.
go=yes
if(!defpac("focas")) {go=no}
if(!defpac("artdata")) {go=no}
if(!deftask("imswap")) {go=no}
if(!deftask("fileroot")) {go=no}
if(!deftask("filecalc")) {go=no}
if(go) {

print ("This tasks requires existing image1.cat, image2.cat, image3.cat, & sum.cat,")
print ("and also requires the following header keywords in the input images:")
print ("rlambda, ebv, gain, rdnoise, fwhmpix, & zeropt.  You have been warned!")

fileroot(image1, validim+)
	img1=fileroot.root
fileroot(image2, validim+)
	img2=fileroot.root
fileroot(image3, validim+)
	img3=fileroot.root
norm1=untrim1
norm2=untrim2
norm3=untrim3
msk=mask
fileroot(sum, validim+)
	ssum=fileroot.root
color_21=color21
color_31=color31
llimbin1=mlim1
	llimbin2=llimbin1+color_21
	llimbin3=llimbin1+color_31
sspacing=spacing
nnbins=nbins
nnobjs=nobjs
nnrepeat=nrepeat
ppixscale=pixscale

ddis=disimages
logname=log_file
	if(logname=="")logname="dev$null"
bborder=border
cf=cleanfactor
sseed=firstseed
sstar=psf

nrecovsum=0.

print (" ", >> logname)
if(ddis)displ(ssum, 1)

#--get Atot, minus border region
print ("get total area, minus border region")
print ("get total area, minus border region", >> logname)
imdel ("tmp_"//msk, ver-, >& "dev$null")
addring (msk, "tmp_"//msk, ngrow=bborder, badval=0)
imstat ("tmp_"//msk, fields="mean,npix", format-, lower=INDEF, upper=INDEF) | scan(mmean,nnpix)
Atot=(mmean*nnpix*ppixscale**2.)/3600.
print(Atot, >> logname)
print(Atot)
imdel ("tmp_"//msk, ver-)

#--get xminmax for the field
print ("get xminmax for the field", >> logname)
imtranspose (msk//"[*,-*]", "tmp_"//msk)
minmax ("tmp_"//msk, force+, update-, ver-)
s1=minmax.maxpix; x=stridx(",",s1); s2=substr(s1,x+1,strlen(s1)-1)
print (s2) | scan (xmin)
minmax ("tmp_"//msk//"[*,-*]", force+, update-, ver-)
s1=minmax.maxpix; x=stridx(",",s1); s2=substr(s1,x+1,strlen(s1)-1)
print (s2) | scan (xmax)
hsel ("tmp_"//msk, "i_naxis2", yes) | scan (xlong)
xmax=xlong+1-xmax
imdel ("tmp_"//msk, ver-)

#--get yminmax for the field
print ("get yminmax for the field", >> logname)
minmax (msk, force+, update-, ver-)
s1=minmax.maxpix; x=stridx(",",s1); s2=substr(s1,x+1,strlen(s1)-1)
print (s2) | scan (ymin)
minmax (msk//"[*,-*]", force+, update-, ver-)
s1=minmax.maxpix; x=stridx(",",s1); s2=substr(s1,x+1,strlen(s1)-1)
print (s2) | scan (ymax)
hsel (msk, "i_naxis2", yes) | scan (ylong)
ymax=ylong+1-ymax

#--border zones
ymin=ymin+bborder; xmin=xmin+bborder
ymax=ymax-bborder; xmax=xmax-bborder
hedit (msk//","//ssum, "xmin", xmin, add+, ver-, show+, update+)
hedit (msk//","//ssum, "ymin", ymin, add+, ver-, show+, update+)
hedit (msk//","//ssum, "xmax", xmax, add+, ver-, show+, update+)
hedit (msk//","//ssum, "ymax", ymax, add+, ver-, show+, update+)
hedit (msk//","//ssum, "border", bborder, add+, ver-, show+, update+)

#--copy all imgs to temporary imgs, get galactic extinction for that filter,
#  find equal skysig scales for tmp imgs, generate random object x,y positions,
#  match all r(j)k images' catalogs plus the summed image, addring to norm.imgs
del ("_match1.mcat", ver-, >& "dev$null")
match ("_match1.mcat", ssum//".cat", img1//".cat")
#
imdel("tmp_"//img1,ver-,>&"dev$null")
imcopy(img1,"tmp_"//img1,ver-)
#
k=0; kmax=1; f1=1; f="f"
hsel (img1, "itersig", yes) | scan (x)
#
hsel (ssum, "ebv", yes) | scan (ebv)
hsel (img1, "rlambda", yes) | scan (rlambda1)
alambda1=ebv*rlambda1
hedit (img1, "alambda1", alambda1, add+, ver-, update+, show-)
#
imdel ("tmp_"//msk, ver-, >& "dev$null")
imdel ("tmp_"//norm1, ver-, >& "dev$null")
addring (msk, "tmp_"//msk, ngrow=bborder, badval=0)
imar (norm1, "*", "tmp_"//msk, "tmp_"//norm1, ver-, noact-)
#
if(img2!="") {
    match ("_match1.mcat", img2//".cat")
    imdel("tmp_"//img2,ver-,>&"dev$null")
    imcopy(img2,"tmp_"//img2,ver-)
    hsel (img2, "itersig", yes) | scan (y)
    f2=x/y
print("f2="//f2)
print("f2="//f2, >> logname)
    hsel (img2, "rlambda", yes) | scan (rlambda2)
    alambda2=ebv*rlambda2
    kmax=2
    imdel ("tmp_"//norm2, ver-, >& "dev$null")
    imar (norm2, "*", "tmp_"//msk, "tmp_"//norm2, ver-, noact-)
#
    if(img3!="") {
        match ("_match1.mcat", img3//".cat")
        imdel("tmp_"//img3,ver-,>&"dev$null")
        imcopy(img3,"tmp_"//img3,ver-)
	hsel (img3, "itersig", yes) | scan (y)
	f3=x/y
print("f3="//f3)
print("f3="//f3, >> logname)
	hsel (img3, "rlambda", yes) | scan (rlambda3)
	alambda3=ebv*rlambda3
        kmax=3
        imdel ("tmp_"//norm3, ver-, >& "dev$null")
	imar (norm3, "*", "tmp_"//msk, "tmp_"//norm3, ver-, noact-)
    }
}
imdel ("tmp_"//msk, ver-)
del ("_tmprandfile", ver-, >& "dev$null")
total=nnrepeat*nnobjs*nnbins
urand (total, 5, scale=1, >> "_tmprandfile")


#--for each FINAL magnitude bin (index i) after g.e. correction 
#			(i.e. detected mag is g.e. mags fainter)
for (i=1; i<=nnbins; i+=1) {
	mag1=llimbin1-(i-1)*sspacing+alambda1
	if(kmax>=2)mag2=llimbin1-(i-1)*sspacing+alambda2+color_21
	if(kmax>=3)mag3=llimbin1-(i-1)*sspacing+alambda3+color_31
	mag=mag1

#--extract cat of fainter-than-this objs from matched r(j)k+sum mcat
print ("extract cat of objs w/m>"//mag//" in filter #1 from matched r(j)k+sum mcat")
print ("extract cat of objs w/m>"//mag//" in filter #1 from matched r(j)k+sum mcat", >> logname)
	del ("_matchf.*cat", ver-, >& "dev$null")
#--mfilter has problems in batch mode; none of the following work in batch mode.
#--can't use ffilter instead, either.
#       mfilter ("f2 \"M "//mag//" 100\"", < "_match1.mcat", > "_matchf.mcat")
#       mfilter ("f2 'M "//mag//" 100'", < "_match1.mcat", > "_matchf.mcat")
#	s1="f2 \"M "//mag//" 100\""
#       mfilter ("f2", s1, < "_match1.mcat", > "_matchf.mcat")
#      mfilter ("f2", "\"M "//mag//" 100\"", < "_match1.mcat", > "_matchf.mcat")
#	mfilter ("f2", "'M "//mag//" 100'", < "_match1.mcat", > "_matchf.mcat")
	mfilter ("f2 'M "//mag//" 100'", < "_match1.mcat", > "_matchf.mcat")
	mextract (1, < "_matchf.mcat", > "_matchf.cat")
	objects("_matchf.cat")

#--clean summed image of objects w/this flux or less in this filter
print ("clean summed image of objects w/this flux or less in this filter", >> logname)
	imdel ("cl_sum.imh", ver-, >& "dev$null")
	imcopy (ssum, "cl_sum", ver-)
	clean (cf, < "_matchf.cat")
	imswap (ssum, "cl_sum")
	del ("_matchf.*cat", ver-, >& "dev$null")
if(ddis)displ("cl_sum", 2)

#--COMPARISON CAT
print("COMPARISON CAT")
print("COMPARISON CAT", >> logname)
#--rerun focas: create catalog
	del ("cl_sum.ar", ver-, >& "dev$null")
	del ("cl_sum.cat", ver-, >& "dev$null")
	hsel (ssum, "variance", yes) | scan (variance)
	del ("temp.file", ver-, >& "dev$null")
	skysig=sqrt(variance)
	print("cl_sum.cat", " ", "cl_sum", " ", "artdata", " ", skysig, "21", " ", "cl_sum.ar", >> "temp.file")
	!set catpars=`awk -f iraf2unix temp.file`; mysetcathesis $catpars
	del ("temp.file", ver-, >& "dev$null")
#--detect
	detect ("cl_sum.cat", 1, ylong, -5*skysig, 5*skysig)
	sky ("cl_sum.cat")
	evaluate ("cl_sum.cat")
#--splits
	del ("temp.file", ver-, >& "dev$null")
	print("cl_sum.cat", " ", "10", >> "temp.file")
	!set catpars=`awk -f iraf2unix temp.file`; mysetcatsize $catpars
	del ("temp.file", ver-, >& "dev$null")
	splits ("cl_sum.cat")
	del ("cl_sum.sky.imh", ver-, >& "dev$null")
	del ("cl_sum.sky.pix", ver-, >& "dev$null")


#--for j=1 to nnrepeat iterations at each magnitude
	nrecovsum=0.
	ndenomsum=0.
	for (j=1; j<=nnrepeat; j+=1) {
		l=1; del ("_tmpcoo?", ver-, >& "dev$null")
#--create coords+mag list of nnobjs psf objs to add to x,y image region
	    	for (l=1; l<=nnobjs; l+=1) {
       		running=(i-1)*nnrepeat*nnobjs+(j-1)*nnobjs+l
       		fields("_tmprandfile", "1-5", lines=running) | scan (x,y,z,a,b)
		x=x*(xmax-xmin)+xmin; y=y*(ymax-ymin)+ymin
       		z=mag1+sspacing*(z-0.5)
		print (x, y, z, >> "_tmpcoo1")
		if (kmax>=2) {
			a=mag2+sspacing*(a-0.5)
			print (x, y, a, >> "_tmpcoo2")
		}
		if (kmax>=3) 
			b=mag3+sspacing*(b-0.5)
			print (x, y, b, >> "_tmpcoo3")
		}


#--ARTDATA CAT
print ("ARTDATA CAT, filter 1 ("//img1//") mag #"//i//" ("//mag//") iteration #"//j)
print ("ARTDATA CAT, filter 1 ("//img1//") mag #"//i//" ("//mag//") iteration #"//j, >> logname)
#--recreate tmp image 
		imdel ("tmp_"//img1, ver-, >& "dev$null")
		imcopy (img1, "tmp_"//img1, ver-)
#--add objs+noise to img
		hsel ("tmp_"//img1, "zeropt", yes) | scan (zeropt)
		hsel ("tmp_"//img1, "gain", yes) | scan (g)
		hsel ("tmp_"//img1, "rdnoise", yes) | scan (rn)
#print (zeropt, " ", g, " ", rn)
    		hsel ("tmp_"//img1, "fwhmpix", yes) | scan (ffwhmpix)
		mkobjects ("tmp_"//img1, header="", background=0., objects="_tmpcoo1", star=sstar, radius=0.9*(ffwhmpix/2.), beta=2.5, ar=1., dist=1., exptime=1., magzero=zeropt, pa=0., poisson+, gain=g, rdnoise=rn, seed=sseed, comments+, xoff=0., yoff=0.)
#--subtract original image
		imar ("tmp_"//img1, "-", img1, "tmp_"//img1, ver-)
#--scale artdata-only image to equal skysub
		imar ("tmp_"//img1, "*", f1, "tmp_"//img1, ver-)
#--add scaled residual images to summed-cleaned-img
		imdel ("do_img", ver-, >& "dev$null")
		imar ("tmp_"//img1, "+", "cl_sum", "do_img", ver-, noact-)

		if(kmax>=2){
#--recreate tmp image 
			imdel ("tmp_"//img2, ver-, >& "dev$null")
			imcopy (img2, "tmp_"//img2, ver-)
			hsel ("tmp_"//img2, "zeropt", yes) | scan (zeropt)
			hsel ("tmp_"//img2, "gain", yes) | scan (g)
			hsel ("tmp_"//img2, "rdnoise", yes) | scan (rn)
#print (zeropt, " ", g, " ", rn)
    			hsel ("tmp_"//img2, "fwhmpix", yes) | scan (ffwhmpix)
			mkobjects ("tmp_"//img2, header="", background=0., objects="_tmpcoo2", star=sstar, radius=0.9*(ffwhmpix/2.), beta=2.5, ar=1., dist=1., exptime=1., magzero=zeropt, pa=0., poisson+, gain=g, rdnoise=rn, seed=sseed, comments+, xoff=0., yoff=0.)
#--subtract original image
			imar ("tmp_"//img2, "-", img2, "tmp_"//img2, ver-)
#--scale artdata-only image to equal skysub
			imar ("tmp_"//img2, "*", f2, "tmp_"//img2, ver-)
#--add scaled residual images to summed-cleaned-img
		        imar("do_img","+","tmp_"//img2,"do_img",ver-,noact-)
		}

		if(kmax>=3){
#--recreate tmp image 
		    	imdel ("tmp_"//img3, ver-, >& "dev$null")
		    	imcopy (img3, "tmp_"//img3, ver-)
		    	hsel ("tmp_"//img3, "zeropt", yes) | scan (zeropt)
		    	hsel ("tmp_"//img3, "gain", yes) | scan (g)
		    	hsel ("tmp_"//img3, "rdnoise", yes) | scan (rn)
    		    	hsel("tmp_"//img3, "fwhmpix", yes) | scan (ffwhmpix)
		    	mkobjects ("tmp_"//img3, header="", background=0., objects="_tmpcoo3", star=sstar, radius=0.9*(ffwhmpix/2.), beta=2.5, ar=1., dist=1., exptime=1., magzero=zeropt, pa=0., poisson+, gain=g, rdnoise=rn, seed=sseed, comments+, xoff=0., yoff=0.)
#--subtract original image
		    	imar ("tmp_"//img3, "-", img3, "tmp_"//img3, ver-)
#--scale artdata-only image to equal skysub
		    	imar ("tmp_"//img3, "*", f3, "tmp_"//img3, ver-)
#--add scaled residual images to summed-cleaned-img
		    	imar("do_img","+","tmp_"//img3,"do_img",ver-,noact-)
		}

#--mask summed image
		imar ("do_img", "*", msk, "do_img", ver-, noact-)

if(ddis)displ("do_img", 3)
if(ddis)sctv("tmp_"//img1, 4)
#if(ddis)imexam("tmp_"//img1, keeplog+, logfile=logname)

#--rerun focas: create catalog
print("rerun focas & create catalog")
		hsel (ssum, "variance", yes) | scan (variance)
		del ("do_sum.ar", ver-, >& "dev$null")
		del ("do_sum.cat", ver-, >& "dev$null")
		del ("temp.file", ver-, >& "dev$null")
		skysig=sqrt(variance)
		print("do_img.cat", " ", "do_img", " ", "artdata", " ", skysig, "21", " ", "do_img.ar", >> "temp.file")
	       !set catpars=`awk -f iraf2unix temp.file`; mysetcathesis $catpars
#--detect
		detect ("do_img.cat", 1, ylong, -5*skysig, 5*skysig)
		sky ("do_img.cat")
		evaluate ("do_img.cat")
#--splits
		del ("temp.file", ver-, >& "dev$null")
		print("do_img.cat", " ", "10", >> "temp.file")
		!set catpars=`awk -f iraf2unix temp.file`; mysetcatsize $catpars
		del ("temp.file", ver-, >& "dev$null")
		splits ("do_img.cat")
		del ("do_img.sky.imh", ver-, >& "dev$null")
		del ("do_img.sky.pix", ver-, >& "dev$null")

#--create focas cat of artdata objects only
		imar ("tmp_"//img1, "*", msk, "tmp_"//img1, ver-, noact-)
		print("tmp_"//img1//".cat", " ", "tmp_"//img1, " ", "artdata", " ", 0.025*skysig, "6", " ", "tmpimg.ar", >> "temp.file")
	       !set catpars=`awk -f iraf2unix temp.file`; mysetcathesis $catpars
		del ("temp.file", ver-, >& "dev$null")
	    detect ("tmp_"//img1//".cat", 1, ylong, -0.025*skysig, 0.025*skysig)
		sky ("tmp_"//img1//".cat")
		evaluate ("tmp_"//img1//".cat")
		del ("tmp_"//img1//".sky.imh", ver-, >& "dev$null")
		del ("tmp_"//img1//".sky.pix", ver-, >& "dev$null")

#--match summed-altered-img cat w/summed-cleaned-img cat, tossing matches
print ("match summed-altered-img cat w/summed-cleaned-img cat, tossing matches", >> logname)
#if(ddis)review("cl_sum.cat", "do_img.cat", "tmp_"//img1//".cat")
	match ("_match.mcat", "cl_sum.cat", "do_img.cat", "tmp_"//img1//".cat")
		mfilter ("e1", "i2", "i3", < "_match.mcat", > "_matchf.mcat")
		mextract (2, < "_matchf.mcat", > "_do_imgf.cat")
#if(ddis)review("_do_imgf.cat")
		objects ("cl_sum.cat") 
		objects ("do_img.cat") 
		objects ("_do_imgf.cat") 
		objects ("tmp_"//img1//".cat") 
		objects ("_do_imgf.cat") | scan (s1, s2, s3, s4, s5, nrecov)
	      objects ("tmp_"//img1//".cat") | scan (s1, s2, s3, s4, s5, ndenom)

#--delete temporary images & catalogs
		imdel ("do_img", ver-, >& "dev$null")
		imdel ("tmp_"//img1, ver-, >& "dev$null")
		imdel ("tmp_"//img2, ver-, >& "dev$null")
		imdel ("tmp_"//img3, ver-, >& "dev$null")
		del ("tmpimg.ar", ver-, >& "dev$null")
		del ("tmp_"//img1//".cat", ver-, >& "dev$null")
		del ("_tmpcoo?", ver-, >& "dev$null")
		del ("_do_img*.cat", ver-)
		del ("_do_img*.ar", ver-)
		del ("do_img*.cat", ver-)
		del ("do_img*.ar", ver-)
		del ("_match.mcat", ver-)
		del ("_matchf.mcat", ver-)

#--next j (nnrepeat iterations at each magnitude)
		nrecovsum=nrecovsum+nrecov
		ndenomsum=ndenomsum+ndenom
print("nrecov="//nrecov, >> logname)
print("nrecovsum="//nrecovsum, >> logname)
print("ndenomsum="//ndenomsum, >> logname)
	}


#--save info on the ith mag bin
	mnorm=mag1
	if(ndenomsum==0.) {
		frecover=0.
	} else {
		frecover=nrecovsum/ndenomsum
	}
print ("frecover="//frecover, >> logname)
	farray[i,1]=frecover
	farray[i,2]=mnorm	#=mag1=mag
       print ("farray["//i//",1-2]="//farray[i,1]//" "//farray[i,2], >> logname)
	if (kmax>=2) {
		farray[i,3]=mag2
		print ("farray["//i//",3]="//farray[i,3], >> logname)
	}
	if (kmax>=3) {
		farray[i,4]=mag3
		print ("farray["//i//",4]="//farray[i,4], >> logname)
	}

#farray 1	2	3	4
#i=1 frecov mag1=mnorm  mag2	mag3
#i=2 frecov mag1=mnorm  mag2	mag3
#...

#--next magnitude bin i 
	imdel ("cl_sum", ver-, >& "dev$null")
	del ("cl_sum.cat", ver-)
	del ("cl_sum.ar", ver-)
    }


#--for each r(j)k image (index k), compute C(mfinal) for k-mag in filter #k
for (k=1; k<=kmax; k+=1){
      img=img1; norm="tmp_"//norm1; mag=mag1; llimbin=llimbin1; alambda=alambda1
    if(k==2){
      img=img2; norm="tmp_"//norm2; mag=mag2; llimbin=llimbin2; alambda=alambda2
    }
    if(k==3){
      img=img3; norm="tmp_"//norm3; mag=mag3; llimbin=llimbin3; alambda=alambda3
    }
print ("compute C(mfinal) for "//mag//" to "//llimbin//" in filter #"//k)
print ("compute C(mfinal) for "//mag//" to "//llimbin//" in filter #"//k, >> logname)
    if(access("complete_"//img))print("Appending to complete_"//img)
    if(access("comparea_"//img))print("Appending to comparea_"//img)
    print("#"//img, >> "complete_"//img)
    print("#mag    completeness		raw", >> "complete_"//img)
    print("#"//img, >> "comparea_"//img)
    print("#mag    area(sqarcmin)	lost_area", >> "comparea_"//img)

#--take untrim*orig image and subtract from maximum to get deltam at each pixel
    minmax (norm, force+, update-, ver-)
    imar (norm, "-", minmax.maxval, "_tmp_"//norm, ver-, noact-)
    imar ("_tmp_"//norm, "*", "-1.", "_tmp_"//norm, ver-, noact-)
#if(ddis)sctv(norm, 3)
#if(ddis)sctv("_tmp_"//norm, 4)

#--get Area(delm) binned in (spacing) bins centered at 
#--0,1*spacing,2*spacing ... (z1=-0.5*spacing,z2=0.5*spacing)
print("get Area(delm)", >> logname)
    del ("_Alist", ver-, >& "dev$null"); del ("_Zlist", ver-, >& "dev$null")
    binmax=1+int(2.5/sspacing)
    imhistog ("_tmp_"//norm, z1=-0.5*sspacing, z2=(binmax-0.5)*sspacing, binw=sspacing, top_closed=no, listout+, >> "_Alist")
    filecalc ("_Alist", "$1;$2*("//ppixscale//"**2.)/3600.", >> "_Zlist")
    list="_Zlist"
    i=1
    while (fscan(list, deltam, area) !=EOF) {
	print (area, >& "dev$null")
	aarray[i,1]=area
	print (deltam, >& "dev$null")
	aarray[i,2]=minmax.maxval-deltam
	print (deltam, >& "dev$null")
	aarray[i,3]=deltam
print ("aarray["//i//",1-3]="//aarray[i,1]//" "//aarray[i,2]//" "//aarray[i,3], >> logname)
	i=i+1
    }
    imdel ("_tmp_"//norm, ver-)

#aarray	1	2	3	4
#i=1	area	mlim	deltam	tmp
#i=2	area	mlim	deltam	tmp
#...


#--get Area(m) binned in (spacing) bins centered on mreal=mfinal+g.e. mag values
print("get Area(m)", >> logname)
    del ("_Alist", ver-, >& "dev$null"); del ("_Zlist", ver-, >& "dev$null")
    imhistog (norm, z1=llimbin-(nnbins-0.5)*sspacing+alambda, z2=llimbin+0.5*sspacing+alambda, binw=sspacing, listout+, >> "_Alist")
    filecalc ("_Alist", "$1;$2*("//ppixscale//"**2.)/3600.", >> "_Zlist")
    list="_Zlist"
    Asum=0.
    while (fscan(list, mreal, area) !=EOF) {
        mfinal=mreal-alambda
#--"area" is the area which has a limiting magnitude in this bin.
#--"Asum" is the area which has a limiting magnitude in this bin or brighter.
	Asum=Asum+area
#--"Atmp" is the area which has a limiting magnitude fainter than this bin.
        Atmp=Atot-Asum
        print(mfinal, Atmp, area, >> "comparea_"//img)
    }
    del ("_Alist", ver-, >& "dev$null"); del ("_Zlist", ver-, >& "dev$null")


#--one mreal=mfinal+g.e. bin at a time
print("one mreal=mfinal+g.e. bin at a time", >> logname)
#--since m_norm goes in 0.5m steps, m_norm=m_real-delta_m will too
    for (i=1; i<=nnbins; i+=1) {
	mreal=llimbin-(i-1)*sspacing+alambda 	#mreal=mnorm in deepest part
	mfinal=mreal-alambda
	ctmp=0.
print("i="//i, >> logname)

#--for each deltam up to deltam=2.5 (corresponding to 1% of peak exposure time)
#--(A(del_m+0.25,del_m-0.25)/Atot) * f(m_real-delta_m) where
#--f(m_real-delta_m) is f(m_norm) for the m_norm nearest m_real-deltam
	deltamax=1+int(2.5/sspacing)	#--guarantees at least 2.5m above mlim
	for (deltam=0; deltam<=deltamax; deltam+=sspacing) {
		j=1+deltam/sspacing	#--jth array entry = deltam above mlim
		mnorm=mreal-deltam
#--find f(m_real-delta_m) = f(m_norm) for the m_norm nearest m_real-deltam
		index=1
		diff=10.
		for (i2=1; i2<=nnbins; i2+=1) {
		    mtmp=farray[i2,k+1]
		    if ((abs(mtmp-mnorm)) < diff) {
			index=i2
			diff=abs(mtmp-mnorm)
#print ("deltam="//deltam, " i2="//i2, " mtmp="//mtmp, " index="//index, " mnorm="//mnorm, " diff="//diff, >> logname)
		    }
		}
#	--(A(del_m+0.25,del_m-0.25)/Atot) * f(m_real-delta_m) 
		aarray[j,4]=aarray[j,1]*farray[index,1]/Atot
print ("j="//j//": aarray[j,4]="//aarray[j,4]//" aarray[j,1]="//aarray[j,1]//" farray["//index//",1]="//farray[index,1]//" ctmp="//ctmp, >> logname)
#	--sum individual A*f terms to get C(mfinal)=C(mreal)
		ctmp=ctmp+aarray[j,4]
#--next delta_m
	}

#--output C(mfinal)
    ctrue=ctmp
    if (ctmp>=1.) {
	ctmp=1.
    }
    print (mfinal, ctmp, ctrue, >> "complete_"//img)

#--next filter k
    }

#--next mfinal (magnitude) bin i 
}

del ("_tmprandfile", ver-, >& "dev$null")
del ("_match.mcat", ver-, >& "dev$null")
del ("_match1.mcat", ver-, >& "dev$null")
imdel ("tmp_"//norm1, ver-, >& "dev$null")
imdel ("tmp_"//norm2, ver-, >& "dev$null")
imdel ("tmp_"//norm3, ver-, >& "dev$null")

} else {
	print("WARNING: packages FOCAS and/or ARTDATA, and/or tasks")
        print("filecalc, fileroot, and/or imswap are not loaded!")
        print("Please load the appropriate package(s) and restart this task.")
}

list=""
if(echog)beep

end

#--for each field
#    --get x,y limits for that field and filter
#    --get g.e. for each filter
#    --choose each filter's fluxes so m_final (= m_real-g.e.)
#      = e.g. 20.25, 20.75, thus in the deepest part of the image where
#      m_norm1=m_real1, m_norm1 = e.g. 20.25+ge1, 20.75+ge1, ...
#      m_norm2=m_real2, m_norm2 = e.g. 20.25+ge2, 20.75+ge2, ...
#    --find scalings for individual-filter images
#    **for each mag bin: each mag bin represents a different mag in each filter
#        (m1=m1, m2=m1+color21, m3=m1+color31), but should really repeat for
#        each filter since the crowding is different to each filter's limit
#        (i.e. mk<m1 objects not same as mr<m2 or mj<m3)
#      --extract cat of fainter-than-mag-in-filter#1 from matched r(j)k mcat
#      --clean summed img of objects w/this mag bin's flux or less in filter#1
#      --focas on the summed-cleaned-img for comparison cat
#      --for n=1 to 100
#        --add objs + noise to x,y image region in each filter
#        --subtract original image in each filter
#        --scale residual image of each filter, as needed
#        --add scaled residual images to summed-cleaned-img
#        --rerun focas
#        --match new cat w/comparison cat
#      --next n (100 iterations at each flux)
#      --compute f(cts) : fraction of recovered sim'd objs with Ltotal=cts
#      --for each filter k
#        --convert to f(m_norm): fraction recovered with m_norm=zp-2.5log(cts)
#        --since m_norm goes in 0.5m steps, m_norm=m_real-delta_m will too
#        for each (m_real=m_final+g.e.),
#        --get Area(delm) binned in 0.5m bins centered @ 0,0.5;z1=-.25,z2=.25
#        --for each (delta_m) in steps of 0.5 starting at 0, 
#          --multiply (A(del_m+0.25,del_m-0.25)/Atot) * f(m_real-delta_m);
#            f(m_real-delta_m) is f(m_norm) for m_norm nearest m_real-deltam
#          --sum individual A*f terms to get C(m_final)=C(m_real-g.e.)
#        --next delta_m
#      --next filter
#    --next flux bin 
#--next field

