# mkrgb -- Make Sun rgb rasterfiles from IRAF images
#
# -- needs "color" package
# -- needs "iterstat" previously run on images, and "zeropt" keyword in headers
#    (note that "zeropt" has the opposite sign than typ. used in phot. solutions
# -- defaults are neg=0 pos=8 magr=19 magg=21 magb=23 log- 
#
# 970712: created by Pat Hall
#
# -- add in (optional) (interactive) looping

procedure mkrgb(red,green,blue,neg,pos,rgb)

string red	{prompt = "Input image for the red component"}
string green	{prompt = "Input image for the green component"}
string blue	{prompt = "Input image for the blue component"}
real neg	{0,prompt = "Number of sigma for minimum intensity"}
real pos	{8,prompt = "Number of sigma for maximum intensity"}
string rgb	{prompt = "Output root name for Sun 24-bit RGB rasterfiles"}

real magred	{19.,prompt = "Maximum mag/sqarcsec for red-gun image"}
real maggreen	{21.,prompt = "Maximum mag/sqarcsec for green-gun image"}
real magblue    {23.,prompt = "Maximum mag/sqarcsec for blue-gun image"}
real pixfact	{2.6,prompt = "2.5*log10((pixels/arcsec)**2."}
bool gogreen	{no,prompt = "Independent green-gun image?"}
bool orient	{yes,prompt = "Preserve orientation of IRAF images?"}
bool log	{no,prompt = "Use logarithmic intensity mapping?"}
#bool interact	{prompt = "Interactive parameter adjustment?"}
string logfile  {"",prompt = "Logfile of rgbsun commands; null string=no logging"}
bool   echog        {yes,prompt="Beep when task is done?"}

begin

string imgred, imgblue, imggreen, rgbroot
real sigred, siggreen, sigblue, zpred, zpgreen, zpblue
real maxred, maxblue, maxgreen
real mag_red, mag_blue, mag_green, negsig, possig

if(orient) {
	imgred=red//"[*,-*]"; imgblue=blue//"[*,-*]"; imggreen=green//"[*,-*]"
} else {
	imgred=red; imgblue=blue; imggreen=green
}
mag_red=magred; mag_blue=magblue; mag_green=maggreen
negsig=neg; possig=pos; rgbroot=rgb

print ("If error, check that images have had iterstat previously run on them.")


# Faintest-object print
hsel (imgred, "itersig", yes) | scan (sigred)
hsel (imggreen, "itersig", yes) | scan (siggreen)
hsel (imgblue, "itersig", yes) | scan (sigblue)

rgbsun (imgred, imggreen, imgblue, rgbroot//"_faint", rz1=negsig*sigred, rz2=possig*sigred, gz1=negsig*siggreen, gz2=possig*siggreen, bz1=negsig*sigblue, bz2=possig*sigblue, logmap=log)

if (logfile!="") {
print ("rgbsun "//imgred//" "//imggreen//" "//imgblue//" "//rgbroot//"_faint rz1="//negsig*sigred//" rz2="//possig*sigred//" gz1="//negsig*siggreen//" gz2="//possig*siggreen//" bz1="//negsig*sigblue//" bz2="//possig*sigblue//" logmap="//log, >> logfile)
}


# Common magnitude scale print
hsel (imgred, "zeropt", yes) | scan (zpred)
hsel (imgblue, "zeropt", yes) | scan (zpblue)
maxred=10**(-0.4*(mag_red+pixfact-zpred))
maxblue=10**(-0.4*(mag_blue+pixfact-zpblue))
if (gogreen) {
	hsel (imggreen, "zeropt", yes) | scan (zpgreen)
	maxgreen=10**(-0.4*(mag_green+pixfact-zpgreen))
} else {
	maxgreen=maxred+maxblue
}

rgbsun (imgred, imggreen, imgblue, rgbroot//"_scale", rz1=0, rz2=maxred, gz1=0, gz2=maxgreen, bz1=0, bz2=maxblue, logmap=log)

if (logfile!="") {
print ("rgbsun "//imgred//" "//imggreen//" "//imgblue//" "//rgbroot//"_scale rz1=0 rz2="//maxred//" gz1=0 gz2="//maxgreen//" bz1=0 bz2="//maxblue//" logmap="//log, >> logfile)
}

print("Display "//rgbroot//"_faint and "//rgbroot//"_scale with xview.")
if(!orient){
	print("Be sure to flip up/down before saving as PostScript")
	print("to preserve IRAF image orientation.")
}

if(echog)beep

end

