procedure scab(imlist)

# scab -- scab over bleeding columns
# 980807: created
# 980809: fixed so that no pixel subscripts should ever be out of bounds 
# 980923: added imexam option
# 990608: added CLOTDONE header card
# 000811: renamed scab.cl from clot.cl ; changed header card name to SCABDONE
#         (but maintained compatibility with CLOTDONE)
# 000813: if a saturated area ends while more than 1 pixel wide
#	  and then starts up again after only 1 row, wacky hijinks ensue.
# **speed up
# **add mask
# **below-sky? (see below)

string    imlist        {prompt="Images on which to work (im1,im2,im3)"}
string    prefix="scab_"    {prompt="Prefix for bleeding star lists for images"}
int    strip_width=9    {prompt="Default width of strip around bleeding stars"}
real    uplimit=4000    {prompt="Minimum value above mean for bleeding columns"}
real    bleedlimit   {prompt="Minimum value found in bleeding columns",mode="q"}
bool    dodisp=yes     {prompt="Display images for imexam?"}
bool    doimexam=yes     {prompt="Interactively mark bleeding stars w/imexam?"}
bool    doscab=yes     {prompt="Do interpolation (scabbing) step?"}
bool    interact=no     {prompt="Interactively select bleeding limits?"}
bool    findisp=yes    {prompt="Display before & after images when done?"}
bool    echog=yes    {prompt="Beep when task is done?"}
struct     *list
struct     *list2

begin

string img, sat_list, im_age, pre_fix, im_list, image, satlist, oldsatlist
int n, nn, x, y, xleft, xright, m, mm, xstar, ystar, xcen, ymin, ymax, xstep, i
int xmin, xmax, stripwidth, xlimleft, xlimright, xtmp, ytmp, xhitmp, xlotmp, l
int frame, xleftstart, xrightstart, xleftold, xrightold
int xleftlim, xrightlim
real leftval, rightval, bleed_limit, pixval, scabval, mean, sig, val_z1, val_z2
real up_limit, oldpixval, testpixval
bool fin_disp, inter_act, do_imexam, do_disp, do_scab

im_list = mktemp("imlist")
sections (imlist, >> im_list)
pre_fix = prefix
stripwidth = strip_width
up_limit = uplimit
do_disp = dodisp
do_imexam = doimexam
do_scab = doscab
inter_act = interact
fin_disp = findisp

if (do_disp) {
    list=im_list
    frame=1
    while (fscan (list,img) != EOF) {
        fileroot(img,validim+)
        im_age=fileroot.root
        imgets (im_age, "itermean")
        if ((imgets.value)!="0") {
            mean=real(imgets.value)
            imgets (im_age, "itersig")
                if ((imgets.value)!="0") {
                        sig=real(imgets.value)
                        val_z1=mean-4*sig
                        val_z2=mean+up_limit
                        displ(im_age,frame,zr-,zs-,z1=val_z1,z2=val_z2)
                } else {
                        beep
                        print ("ITERSIG not found for "//im_age)
                        display(im_age,frame)
                }
        } else {
                beep
                print ("ITERMEAN not found for "//im_age)
                    display(im_age,frame)
        }
    frame=frame+1
    }    
}    
if (do_imexam) {
    list=im_list; satlist=""; oldsatlist=""
    while (fscan (list,img) != EOF) {
        fileroot(img,validim+)
        im_age=fileroot.root
        satlist=pre_fix//im_age
        if (access(oldsatlist)) {
            type (oldsatlist, >> satlist)
            type (oldsatlist)
        }
        imexam (im_age, keeplog+, logf=satlist)
        vi ("-c ':g/$/s// "//stripwidth//"/g'", satlist)
        oldsatlist=satlist
    }
}
if (do_scab) {
# do each saturated star list
    list=im_list
    while (fscan (list,image) != EOF) {
        imgets (image, "CLOTDONE", >& "dev$null")
        if (imgets.value=="yes") {
            hedit(img, "SCABDONE", "yes", add+, ver-, show-, upd-)
        }
        imgets (image, "SCABDONE", >& "dev$null")
        if (imgets.value!="yes" && !imaccess("_orig"//image)) {
            print("Now scabbing over image "//image)
            fileroot(image,validim+)
            im_age=fileroot.root
            satlist=pre_fix//im_age
            img = "_tmp"//image
# get approx coords of saturated stars (x align more impt than y align)
            list2=satlist
            l=0
            while (fscan (list2,xstar,ystar,stripwidth) != EOF) {
                m = stripwidth; mm=(m-1)/2
                l = l+1

                    if (stripwidth != 0) {
# imhistogram m-pixel-wide strip centered on star's x position
                        print("Working on strip for star #"//l//" ("//xstar//","//ystar//")")
                        if (l==1) {
                            xmin=1; ymin=1
                            imgets (image, "i_naxis1")
                            xmax=real(imgets.value)
                            imgets (image, "i_naxis2")
                            ymax=real(imgets.value)
                            mean=0
                            imgets (image, "itermean")
                            if((imgets.value)!="0") mean=real(imgets.value)
                        }
                        if (l==1 || inter_act) {
                            xlotmp=max(xstar-mm,xmin); xhitmp=min(xstar+mm,xmax)
                            imhistogram (image//"["//xlotmp//":"//xhitmp//",*]", z1=INDEF, z2=INDEF, binwidth=INDEF, nbins=512, autoscale+, top_closed-, hist_type="normal", listout-, plot_type="line", logy+, device="stdgraph")
# select lower bleedcol cutoff (= bleed_limit)
                            if (inter_act) {
                                beep
                                bleed_limit=bleedlimit
                            } else {
                                bleed_limit=mean+up_limit
                                print("Minimum bleeding column value is "//bleed_limit)
                            }
                        }
                        if (l==1) {
                            print ("Copying to temporary working image...")
                            imdel (img, ver-, >& "dev$null"); imcopy (image, img, ver-)
                        }

# --n=1: start with m-pixel-wide strip centered on star x,y
# --n=1: check central pixel and find xleft & xright
# --n>1: if xleft=xright check for > bleed_limit & you're done
#        check xleft: if < bleedlimit 
#        keep checking right until > bleedlimit or until xright;
#     if xleft < xright then do same going left from xright
#        save xleft & xright for next iteration

                        for (y=ystar; y<=ymax; y+=1) {
# --n=1: start with m-pixel-wide strip centered on star x,y
                            if (y==ystar) {
                                print ("Scabbing...")
                                xcen=xstar; xleft=xcen-mm; xright=xcen+mm; leftval=0; rightval=0
# --check central pixel
                                imstat(img//"["//xcen//":"//xcen//","//y//":"//y//"]", fields="mean", format-) | scan (pixval)
# if < bleed_limit check the pixels immediately to the left & right;
# if either of them are > bleed_limit; go to "> bleed_limit" section
# if neither are > bleed_limit, this half of the star is finished
                                if (pixval < bleed_limit) {
                                    xlotmp=max(xcen-1,xmin); xhitmp=min(xcen+1,xmax)
                                    xstep=xhitmp-xlotmp
                                    for (x=xlotmp; x<=xhitmp; x+=xstep) {
                                        imstat(img//"["//x//":"//x//","//y//":"//y//"]", fields="mean", format-) | scan (pixval)
                                        if (pixval > bleed_limit) xcen=x
                                        if (pixval > bleed_limit) break
                                        if (x==xhitmp) y=ymax+1
                                    }
                                }
# --if > bleed_limit check pixels to the left until < bleed_limit;
                                if (pixval >= bleed_limit) {
                                    xlimleft=max(xcen-mm,xmin); xlotmp=max(xcen-1,xmin)
                                    for (x=xlotmp; x>=xlimleft-1; x-=1) {
                                        imstat(img//"["//x//":"//x//","//y//":"//y//"]", fields="mean", format-) | scan (pixval)
                                        if (pixval < bleed_limit) {
                                            leftval = pixval; xleft = x+1
                                        }
                                        if (pixval < bleed_limit) break
                                        if (x==xmin) {
                                            leftval = bleed_limit; xleft = xmin
                                        }
                                        if (x==xmin) break
                                    }
# --repeat with pixels to the right; 
                                    xlimright=min(xcen+mm,xmax); xhitmp=min(xcen+1,xmax)
                                    for (x=xhitmp; x<=xlimright+1; x+=1) {
                                        imstat(img//"["//x//":"//x//","//y//":"//y//"]", fields="mean", format-) | scan (pixval)
                                        if (pixval < bleed_limit) {
                                            rightval = pixval; xright = x-1
                                        }
                                        if (pixval < bleed_limit) break
                                        if (x==xmax)  {
                                            rightval = bleed_limit; xright = xmax
                                        }
                                        if (x==xmax) break
                                    }
                                }
# replace all pixels in the row's bleed region with 0.5*(leftval+rightval)
                                if ((leftval != 0) && (rightval != 0)) {
                                    scabval=0.5*(leftval+rightval)
                                    imreplace(img//"["//xleft//":"//xright//","//y//":"//y//"]", scabval, lower=INDEF, upper=INDEF)
                                }
# --save xleft & xright for next iteration
                                xrightstart=xright ; xleftstart=xleft
                                xrightold=xright ; xleftold=xleft

# --n>1: 
                            } else {
    
                                xcen=xstar; xleft=xleftold; xright=xrightold; leftval=0; rightval=0
# --n>1: if xleftold=xrightold check for > bleed_limit & you're done
                                if (xleftold==xrightold) {
                                    imstat(img//"["//xleftold//":"//xleftold//","//y//":"//y//"]", fields="mean", format-) | scan (pixval)
                                    if (pixval >= bleed_limit) {
                                        imreplace(img//"["//xleftold//":"//xleftold//","//y//":"//y//"]", scabval, lower=INDEF, upper=INDEF)
                                    } else {
                                        y=ymax+1
                                    }
                                } else {
    
# check xleftold
                                    imstat(img//"["//xleftold//":"//xleftold//","//y//":"//y//"]", fields="mean", format-) | scan (testpixval)
# if >= bleedlimit ...
                                    if (testpixval >= bleed_limit) {
# ... keep checking left until xlimleft or < bleedlimit 
# stay within image boundaries
                                        xlimleft=max(xcen-mm,xmin); xlotmp=max(xleft-1,xmin)
                                        for (x=xlotmp; x>=xlimleft; x-=1) {
                                            imstat(img//"["//x//":"//x//","//y//":"//y//"]", fields="mean", format-) | scan (pixval)
# if < bleed_limit, set new xleft
                                            if (pixval < bleed_limit) {
                                                leftval = pixval; xleft = x+1
                                            }
                                            if (pixval < bleed_limit) break
                                            if (x==xlimleft) {
                                                leftval = bleed_limit; xleft = xlimleft
                                            }
                                            if (x==xlimleft) break
                                        }
# if < bleedlimit ...
                                    } else {
# ... keep checking right until xright or > bleedlimit 
					oldpixval = testpixval
                                        xlimright=min(xright,xmax); xlotmp=min(xleft+1,xmax)
                                        for (x=xlotmp; x<=xlimright; x+=1) {
                                            imstat(img//"["//x//":"//x//","//y//":"//y//"]", fields="mean", format-) | scan (pixval)
                                            if (pixval > bleed_limit) {
                                                leftval = oldpixval; xleft = x-1
                                            }
                                            if (pixval > bleed_limit) break
                                            if (x==xlimright)  {
                                                leftval=bleed_limit; xleft=xlimright
                                            }
                                            if (x==xlimright) break
					    oldpixval = pixval
                                        }
                                    }
#     if xleft < xright then do same for rightval
                                    if (xleft < xrightold) {
# check xrightold
                                        imstat(img//"["//xrightold//":"//xrightold//","//y//":"//y//"]", fields="mean", format-) | scan (testpixval)
# if >= bleedlimit ...
                                        if (testpixval >= bleed_limit) {
# ... keep checking right until xlimright or < bleedlimit 
# stay within image boundaries
                                            xlimright=min(xcen+mm,xmax); xlotmp=min(xright+1,xmax)
                                            for (x=xlotmp; x<=xlimright; x+=1) {
                                                imstat(img//"["//x//":"//x//","//y//":"//y//"]", fields="mean", format-) | scan (pixval)
# if < bleed_limit, set new xright
                                                if (pixval < bleed_limit) {
                                                    rightval = pixval; xright = x-1
                                                }
                                                if (pixval < bleed_limit) break
                                                if (x==xlimright) {
                                                    rightval = bleed_limit; xright = xlimright
                                                }
                                                if (x==xlimright) break
                                            }
# if < bleedlimit ...
                                        } else {
# ... keep checking left until xleft or > bleedlimit 
					    oldpixval = testpixval
                                            xlimleft=max(xleft,xmin); xlotmp=max(xright-1,xmin)
                                            for (x=xlotmp; x>=xlimleft; x-=1) {
                                                imstat(img//"["//x//":"//x//","//y//":"//y//"]", fields="mean", format-) | scan (pixval)
                                                if (pixval > bleed_limit) {
                                                    rightval = oldpixval; xright = x+1
                                                }
                                                if (pixval > bleed_limit) break
                                                if (x==xlimleft)  {
                                                    rightval=bleed_limit; xright = xlimleft
                                                }
                                                if (x==xlimleft) break
					        oldpixval = pixval
                                            }
                                        }
                                    }
# --replace all pixels in the row's bleed region with 0.5*(leftval+rightval)
                                if ((leftval != 0) && (rightval != 0)) {
                                    scabval=0.5*(leftval+rightval)
                                    imreplace(img//"["//xleft//":"//xright//","//y//":"//y//"]", scabval, lower=INDEF, upper=INDEF)
                                }
# --save xleft & xright for next iteration
                                xrightold=xright ; xleftold=xleft
# --end of xleft!=xright branch (218)
                                }
# --end of n>1 subloop in upward loop (207)
                            }
			    #print (y//" "//xleft//" "//xright//" "//leftval//" "//rightval)
# --next row (next y) (153) 
                        } 
# --now work downwards along the edge of the replacement region
                        ytmp=max(ystar-1,ymin)
                        for (y=ytmp; y>=ymin; y-=1) {
# --n=1: start w/same left & right limits as upwards branch did
			    if (y==ytmp) { xleftold=xleftstart; xrightold=xrightstart }
                            xcen=xstar; xleft=xleftold; xright=xrightold; leftval=0; rightval=0
# --n>1: if xleftold=xrightold check for > bleed_limit & you're done
                            if (xleftold==xrightold) {
#print("a")
                                imstat(img//"["//xleftold//":"//xleftold//","//y//":"//y//"]", fields="mean", format-) | scan (pixval)
                                if (pixval >= bleed_limit) {
                                    imreplace(img//"["//xleftold//":"//xleftold//","//y//":"//y//"]", scabval, lower=INDEF, upper=INDEF)
                                } else {
                                    y=ymin-1
                                }
                            } else {
#print("b")
# check xleftold
                                imstat(img//"["//xleftold//":"//xleftold//","//y//":"//y//"]", fields="mean", format-) | scan (testpixval)
# if >= bleedlimit ...
                                if (testpixval >= bleed_limit) {
#print("c")
# ... keep checking left until xmin or < bleedlimit 
# stay within image boundaries
                                    xlimleft=max(xcen-mm,xmin); xlotmp=max(xleft-1,xmin)
                                    for (x=xlotmp; x>=xlimleft; x-=1) {
                                        imstat(img//"["//x//":"//x//","//y//":"//y//"]", fields="mean", format-) | scan (pixval)
# if < bleed_limit, set new xleft
                                        if (pixval < bleed_limit) {
                                            leftval = pixval; xleft = x+1
                                        }
                                        if (pixval < bleed_limit) break
                                        if (x==xlimleft) {
                                            leftval = bleed_limit; xleft = xlimleft
                                        }
                                        if (x==xlimleft) break
                                    }
# if < bleedlimit ...
                                } else {
#print("d")
# ... keep checking right until xright or > bleedlimit 
				    oldpixval = testpixval
                                    xlimright=min(xright,xmax); xlotmp=min(xleft+1,xmax)
                                    for (x=xlotmp; x<=xlimright; x+=1) {
                                        imstat(img//"["//x//":"//x//","//y//":"//y//"]", fields="mean", format-) | scan (pixval)
                                        if (pixval > bleed_limit) {
                                            leftval = oldpixval; xleft = x-1
                                        }
                                        if (pixval > bleed_limit) break
                                        if (x==xlimright)  {
                                            leftval=bleed_limit; xleft=xlimright
                                        }
                                        if (x==xlimright) break
					oldpixval = pixval
                                    }
                                }
#     if xleft < xright then do same for rightval
                                if (xleft < xrightold) {
#print("e")
# check xrightold
                                    imstat(img//"["//xrightold//":"//xrightold//","//y//":"//y//"]", fields="mean", format-) | scan (testpixval)
# if >= bleedlimit ...
                                    if (testpixval >= bleed_limit) {
#print("f")
# ... keep checking right until xmax or < bleedlimit 
# stay within image boundaries
                                        xlimright=min(xcen+mm,xmax); xlotmp=min(xright+1,xmax)
                                        for (x=xlotmp; x<=xmax; x+=1) {
                                            imstat(img//"["//x//":"//x//","//y//":"//y//"]", fields="mean", format-) | scan (pixval)
# if < bleed_limit, set new xright
                                            if (pixval < bleed_limit) {
                                                rightval = pixval; xright = x-1
                                            }
                                            if (pixval < bleed_limit) break
                                            if (x==xlimright) {
                                                rightval = bleed_limit; xright = xlimright
                                            }
                                            if (x==xlimright) break
                                        }
# if < bleedlimit ...
                                    } else {
#print("g")
# ... keep checking left until xleft or > bleedlimit 
				        oldpixval = testpixval
                                        xlimleft=max(xleft,xmin); xlotmp=max(xright-1,xmin)
                                        for (x=xlotmp; x>=xlimleft; x-=1) {
                                            imstat(img//"["//x//":"//x//","//y//":"//y//"]", fields="mean", format-) | scan (pixval)
                                            if (pixval > bleed_limit) {
                                                rightval = oldpixval; xright = x+1
                                            }
                                            if (pixval > bleed_limit) break
                                            if (x==xlimleft)  {
                                                rightval=bleed_limit; xright = xlimleft
                                            }
                                            if (x==xlimleft) break
					    oldpixval = pixval
                                        }
                                    }
# -- end of "xleft < xrightold" loop (368)
                                }
# --replace all pixels in the row's bleed region with 0.5*(leftval+rightval)
                                if ((leftval != 0) && (rightval != 0)) {
                                    scabval=0.5*(leftval+rightval)
                                    imreplace(img//"["//xleft//":"//xright//","//y//":"//y//"]", scabval, lower=INDEF, upper=INDEF)
                                }
# --save xleft & xright for next iteration
                                xrightold=xright ; xleftold=xleft
# --end of xleftold!=xrightold branch (328)
                            }
			    #print (y//" "//xleft//" "//xright//" "//leftval//" "//rightval)
# --next row (next y) (318)
                        }
# --end of "if stripwidth != 0" loop (115) [extra indentation]
                    }

# **also repeat using min_sky instead of bleed_limit for oversubtracted regions
#   in SNe images, reversing the sign so to speak

# --repeat for next star in satlist (111)
            }
            hedit(img, "SCABDONE", "yes", add+, ver-, show+, upd+)
            if (fin_disp) {
                if(imaccess(img)){
                    display (image, 1)
                    display (img, 2)
                }
            }
            if(imaccess(img)){
                imren (image, "_orig"//image, ver+)
                imren (img, image, ver+)
            }
# --end of "Now scabbing over" loop (102)
        } else {
            print("SCABDONE header card and/or _orig"//image//" already exist;")
            print("scabbing already done on "//image//" -- not redone.")
        }
# --repeat for next satlist (next list in im_list) (96)
    }
# --end of do_scab loop (93)
}

del (im_list, ver-)

if (echog) beep

end


